library(data.table)
library(Hmisc)
library(ppcor)
library(nlme)
library(plyr)
library(purrr)
library(lavaan)

#DICVOVERING DIRECTIONS of LINKS WITH CORRELATION ANALYSIS
{
  #Population Death Model
  {
    #read data 
    {data1<-t(data.table(read.csv("PopulationDeathModel.csv",header=T)))
    colnames(data1) = data1[1, ] # the first row will be the header
    data1 = data1[-1, ] 
    data1=data1[-nrow(data1),]
    row.names(data1) <- 1:nrow(data1)
    #View(data1)
    data1<-data.frame(data1)
    
    data1$Deaths <- as.numeric(levels(data1$Deaths))[data1$Deaths]
    data1$Population <- as.numeric(levels(data1$Population))[data1$Population]
    
    x<-data1$Deaths
    y<- data1$Population
    
    #Rate of change of Population
    dy<- lapply(1:(length(y)), function(i) (y[i+1]-y[i]))   
    dy<- dy[-length(dy)]
    x2<- x[-c( length(x))] 
    dy<- apply( cbind( dy ) , 1 , unlist )
    
    }
    #PLOT 
    {
      plot(y,col=1, type="l", ylim=c(0, max(y)))
      plot(x,col=1, type="l", ylim=c(0, max(x)))
      plot(dy,col=1, type="l")
    }
    
    #Direct correlation analysis between Death and Population
    rcorr(x,y, type="spearman") 
    #Through flow, cause effect correlation analysis
    rcorr(x2,dy, type="spearman") 
    }
  
  #Population Birth and Death Model (without noise)
  {
    #read data 
    {
      ## Birth Fraction <Death Fraction
      { data1<-t(data.table(read.csv(paste0("PopulationBirthDeathModel",".csv"),header=T),header=T))
      }
      
      colnames(data1) = data1[1, ] # the first row will be the header
      data1 = data1[-1, ] 
      data1=data1[-nrow(data1),]
      row.names(data1) <- 1:nrow(data1)
      #View(data1)
      data1<-data.frame(data1)
      
      data1$Deaths <- as.numeric(levels(data1$Deaths))[data1$Deaths]
      data1$Population <- as.numeric(levels(data1$Population))[data1$Population]
      data1$Births<- as.numeric(levels(data1$Births))[data1$Births]
      
      x<-data1$Deaths
      y<- data1$Population 
      z<- data1$Births
      #Rate of change of Population=dy
      dy<- lapply(1:(length(y)), function(i) (y[i+1]-y[i]))   #LLA: (y[i+1]-y[i-1])/2
      dy<- dy[-length(dy)]
      x2<- x[-c( length(x))] 
      z2<- z[-length(z)]
      dy<- apply( cbind( dy ) , 1 , unlist )
    
    }
    #PLOT 
    {
      plot(y,col=1, type="l", ylim=c(0, max(y)))
      plot(x,col=1, type="l", ylim=c(0, max(x)))
      plot(z,col=1, type="l", ylim=c(0, max(z)))
      plot(dy,col=1, type="l")
    }
    
    #Direct correlation analysis between Death and Population
    c<- cbind(x,z)
    rcorr(c,y, type="spearman") 
    
    #Through flow, cause effect correlation analysis
    rcorr(x2,dy, type="spearman") 
    c<- cbind(x2,z2)
    rcorr(c,dy, type="spearman") 
    #Partial correlation between Death and Rate of Change of population
    pcor.test(z2,dy, x2, method="spearman")
    #Partial correlation between Birth and Rate of Change of population
    pcor.test(x2,dy, z2, method="spearman")
    
  }
  
  #Population Birth and Death Model (with noise)
  {
    #read data 
    {
      ## Noisy Birth Fraction < Noisy Death Fraction
      { data1<-t(data.table(read.csv(paste0("NoisyPopulationBirthDeathModel",".csv"),header=T),header=T))
      }
      
      colnames(data1) = data1[1, ] # the first row will be the header
      data1 = data1[-1, ] 
      data1=data1[-nrow(data1),]
      row.names(data1) <- 1:nrow(data1)
      #View(data1)
      data1<-data.frame(data1)
      
      data1$Deaths <- as.numeric(levels(data1$Deaths))[data1$Deaths]
      data1$Population <- as.numeric(levels(data1$Population))[data1$Population]
      data1$Births<- as.numeric(levels(data1$Births))[data1$Births]
      
      x<-data1$Deaths
      y<- data1$Population 
      z<- data1$Births
      #Rate of change of Population=dy
      dy<- lapply(1:(length(y)), function(i) (y[i+1]-y[i]))   #LLA: (y[i+1]-y[i-1])/2
      dy<- dy[-length(dy)]
      x2<- x[-c( length(x))] 
      z2<- z[-length(z)]
      dy<- apply( cbind( dy ) , 1 , unlist )
      
    }
    #PLOT 
    {
      plot(y,col=1, type="l", ylim=c(0, max(y)))
      plot(x,col=1, type="l", ylim=c(0, max(x)))
      plot(z,col=1, type="l", ylim=c(0, max(z)))
      plot(dy,col=1, type="l")
    }
    
    #Direct correlation analysis between Death and Population
    c<- cbind(x,z)
    rcorr(c,y, type="spearman") 
    
    #Through flow, cause effect correlation analysis
    rcorr(x2,dy, type="spearman") 
    c<- cbind(x2,z2)
    rcorr(c,dy, type="spearman") 
    #Partial correlation between Death and Rate of Change of population
    pcor.test(z2,dy, x2, method="spearman")
    #Partial correlation between Birth and Rate of Change of population
    pcor.test(x2,dy, z2, method="spearman")
    
  }
}

#DISCOVERING STOCK FLOW VARIBLES CURVE FITTING
{ 
  source("curve fit over all.R")
  
  #Population Death Model
  {
    #read data 
    {data1<-t(data.table(read.csv("PopulationDeathModel.csv",header=T)))
    colnames(data1) = data1[1, ] # the first row will be the header
    data1 = data1[-1, ] 
    data1=data1[-nrow(data1),]
    row.names(data1) <- 1:nrow(data1)
    #View(data1)
    data1<-data.frame(data1)
    
    data1$Deaths <- as.numeric(levels(data1$Deaths))[data1$Deaths]
    data1$Population <- as.numeric(levels(data1$Population))[data1$Population]
    
    x<-data1$Deaths
    y<- data1$Population
    
    #Rate of change of Population
    dy<- lapply(1:(length(y)), function(i) (y[i+1]-y[i]))   #LLA: (y[i+1]-y[i-1])/2
    dy<- dy[-length(dy)]
    x2<- x[-c( length(x))] 
    dy<- apply( cbind( dy ) , 1 , unlist )
    
    }
    #PLOT 
    {
      plot(y,col=1, type="l", ylim=c(0, max(y)))
      plot(x,col=1, type="l", ylim=c(0, max(x)))
      plot(dy,col=1, type="l")
    }
    #Applying Curve Fit
    {
      fit1<- curveFitAll(x,y)
      fit2<- curveFitAll(x=y,y=x)
      #FOR FIT1
      {results<- as.data.frame(fit1[[1]])
        #sigmaCol<- results[, which(colnames(results)=="Sigma")]
        #sigmaCol<- as.numeric(levels(sigmaCol))[sigmaCol]
        #results[, which(colnames(results)=="Sigma")]<- round(sigmaCol, digits= 2)
        results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
        results$Sigma<- round(results$Sigma, digits= 4)
        results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
        results$Mape<- round(results$Mape, digits= 4)
        
        Pareto1 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
        Pareto1
        all<-data.frame(Sigma=Pareto1$Sigma,Mape=Pareto1$Mape,Method=Pareto1$Method,Function=Pareto1$Function, Relation=Pareto1$Relation)
        all
        # best<-data.frame(Sigma=Pareto1[1,]$Sigma,Mape=Pareto1[1,]$Mape,Method=Pareto1[1,]$Method,Function=Pareto1[1,]$Function, Relation=Pareto1[1,]$Relation)
        # best
        
        predictions<- as.data.frame(fit1[[2]])
        bestindex<-as.numeric(rownames(Pareto1[1,]))
        yestimate<-predictions[,as.numeric(rownames(Pareto1[1,]))]
        yestimate<-as.matrix(yestimate)
      }
      # PLOT for FIT1
      { plot(x,y, type="l", ylim=c(min(min(yestimate[,1]),min(y)), max(max(yestimate[,1]),max(y))))
        lines(x,yestimate[,1], col=2)
      }
      #FOR FIT2
      { results<- as.data.frame(fit2[[1]])
        results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
        results$Sigma<- round(results$Sigma, digits= 5)
        results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
        results$Mape<- round(results$Mape, digits= 5)
        
        Pareto2 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
        Pareto2
        all<-data.frame(Sigma=Pareto2$Sigma,Mape=Pareto2$Mape,Method=Pareto2$Method,Function=Pareto2$Function, Relation=Pareto2$Relation)
        all
        # best<-data.frame(Sigma=Pareto2[1,]$Sigma,Mape=Pareto2[1,]$Mape,Method=Pareto2[1,]$Method,Function=Pareto2[1,]$Function, Relation=Pareto2[1,]$Relation)
        # best
         
        predictions<- as.data.frame(fit2[[2]])
        bestindex<-as.numeric(rownames(Pareto2[1,]))
        yestimate<-predictions[,as.numeric(rownames(Pareto2[1,]))]}
      # PLOT FOR FIT2
      {plot(y,x, type="l", ylim=c(min(min(yestimate),min(x)), max(max(yestimate),max(x))))
        lines(y,yestimate, col=2)
      }
    }
    
  }
  #Room Temperature Model
  {
    #read data
    {data1<-t(data.table(read.csv("RoomTemperatureModel.csv",header=T),header=T))
    
    colnames(data1) = data1[1, ] # the first row will be the header
    data1 = data1[-1, ]
    data1=data1[-nrow(data1),]
    row.names(data1) <- 1:nrow(data1)
    View(data1)
    data1<-data.frame(data1)
    
    data1$Temperature.Change <- as.numeric(levels(data1$Temperature.Change))[data1$Temperature.Change]
    data1$Room.Teperature <- as.numeric(levels(data1$Room.Teperature))[data1$Room.Teperature]
    
    x<-data1$Temperature.Change
    y<- data1$Room.Teperature
    
    #Rate of change of Population
    dy<- lapply(1:(length(y)), function(i) (y[i+1]-y[i]))   #LLA: (y[i+1]-y[i-1])/2
    dy<- dy[-length(dy)]
    x2<- x[-c( length(x))] 
    dy<- apply( cbind( dy ) , 1 , unlist )}
    #PLOT 
    {
      plot(y,col=1, type="l", ylim=c(0, max(y)))
      plot(x,col=1, type="l", ylim=c(0, max(x)))
      plot(dy,col=1, type="l")
    }
    #Applying Curve Fit
    {
      fit1<- curveFitAll(x,y)
      fit2<- curveFitAll(x=y,y=x)
      #FOR FIT1
      {results<- as.data.frame(fit1[[1]])
        results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
        results$Sigma<- round(results$Sigma, digits= 5)
        results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
        results$Mape<- round(results$Mape, digits= 5)
        
        Pareto1 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
        Pareto1
        all<-data.frame(Sigma=Pareto1$Sigma,Mape=Pareto1$Mape,Method=Pareto1$Method,Function=Pareto1$Function, Relation=Pareto1$Relation)
        all
        # best<-data.frame(Sigma=Pareto1[1,]$Sigma,Mape=Pareto1[1,]$Mape,Method=Pareto1[1,]$Method,Function=Pareto1[1,]$Function, Relation=Pareto1[1,]$Relation)
        # best
        
        predictions<- as.data.frame(fit1[[2]])
        bestindex<-as.numeric(rownames(Pareto1[1,]))
        yestimate<-predictions[,as.numeric(rownames(Pareto1[1,]))]
        yestimate<-as.matrix(yestimate)
      }
      # PLOT for FIT1
      { plot(x,y, type="l", ylim=c(min(min(yestimate[,1]),min(y)), max(max(yestimate[,1]),max(y))))
        lines(x,yestimate[,1], col=2)
      }
      #FOR FIT2
      { results<- as.data.frame(fit2[[1]])
        results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
        results$Sigma<- round(results$Sigma, digits= 5)
        results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
        results$Mape<- round(results$Mape, digits= 5)
        
        Pareto2 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
        Pareto2
        all<-data.frame(Sigma=Pareto2$Sigma,Mape=Pareto2$Mape,Method=Pareto2$Method,Function=Pareto2$Function, Relation=Pareto2$Relation)
        all
        # best<-data.frame(Sigma=Pareto2[1,]$Sigma,Mape=Pareto2[1,]$Mape,Method=Pareto2[1,]$Method,Function=Pareto2[1,]$Function, Relation=Pareto2[1,]$Relation)
        # best
        
        predictions<- as.data.frame(fit2[[2]])
        bestindex<-as.numeric(rownames(Pareto2[1,]))
        yestimate<-predictions[,as.numeric(rownames(Pareto2[1,]))]}
      # PLOT FOR FIT2
      {plot(y,x, type="l", ylim=c(min(min(yestimate),min(x)), max(max(yestimate),max(x))))
        lines(y,yestimate, col=2)
      }
    }
  } }

#DISCOVERING MATHEMATICAL EXPRESSIONS 
{
  #Room Temperature Model
  {
    #CURVEFITTING
    { 
      source("curve fit over all.R")
      
      
      #Room Temperature Model 
      {
        #read data
        {
          data1<-t(data.table(read.csv("SimplifiedRoomTemperatureModel.csv",header=T),header=T))
          
          colnames(data1) = data1[1, ] # the first row will be the header
          data1 = data1[-1, ]
          data1=data1[-nrow(data1),]
          row.names(data1) <- 1:nrow(data1)
          #View(data1)
          data1<-data.frame(data1)
          
          data1$Temperature.Change <- as.numeric(levels(data1$Temperature.Change))[data1$Temperature.Change]
          data1$Room.Temperature <- as.numeric(levels(data1$Room.Temperature))[data1$Room.Temperature]
          data1$Discrepancy <- as.numeric(levels(data1$Discrepancy))[data1$Discrepancy]
          
          x<-data1$Temperature.Change
          y<- data1$Room.Temperature
          z<-data1$Discrepancy
          #Rate of change of Population
          dy<- lapply(1:(length(y)), function(i) (y[i+1]-y[i]))   #LLA: (y[i+1]-y[i-1])/2
          dy<- dy[-length(dy)]
          x2<- x[-c( length(x))]
          z2<- z[-c( length(z))] 
          dy<- apply( cbind( dy ) , 1 , unlist )
        }
        #PLOT 
        {
          plot(y,col=1, type="l", ylim=c(0, max(y)))
          plot(x,col=1, type="l", ylim=c(0, max(x)))
          plot(dy,col=1, type="l")
        }
        #Applying Curve Fit
        {
          fit1<- curvefit(x=y,y=z)
          fit2<- curvefit(x=z,y=x)
          #FOR FIT1
          {results<- as.data.frame(fit1[[1]])
            results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
            results$Sigma<- round(results$Sigma, digits= 5)
            results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
            results$Mape<- round(results$Mape, digits= 5)
            
            Pareto1 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
            Pareto1
            # all<-data.frame(Sigma=Pareto1$Sigma,Mape=Pareto1$Mape,Method=Pareto1$Method,Function=Pareto1$Function, Relation=Pareto1$Relation)
            # all
            # best<-data.frame(Sigma=Pareto1[1,]$Sigma,Mape=Pareto1[1,]$Mape,Method=Pareto1[1,]$Method,Function=Pareto1[1,]$Function, Relation=Pareto1[1,]$Relation)
            # best
            
            predictions<- as.data.frame(fit1[[2]])
            bestindex<-as.numeric(rownames(Pareto1[1,]))
            yestimate<-predictions[,as.numeric(rownames(Pareto1[1,]))]
            yestimate<-as.matrix(yestimate)
          }
          # PLOT for FIT1
          { plot(x,y, type="l", ylim=c(min(min(yestimate[,1]),min(y)), max(max(yestimate[,1]),max(y))))
            lines(x,yestimate[,1], col=2)
          }
          #FOR FIT2
          { results<- as.data.frame(fit2[[1]])
            results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
            results$Sigma<- round(results$Sigma, digits= 5)
            results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
            results$Mape<- round(results$Mape, digits= 5)
            
            Pareto2 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
            Pareto2
            # all<-data.frame(Sigma=Pareto2$Sigma,Mape=Pareto2$Mape,Method=Pareto2$Method,Function=Pareto2$Function, Relation=Pareto2$Relation)
            # all
            # best<-data.frame(Sigma=Pareto2[1,]$Sigma,Mape=Pareto2[1,]$Mape,Method=Pareto2[1,]$Method,Function=Pareto2[1,]$Function, Relation=Pareto2[1,]$Relation)
            # best
            
            predictions<- as.data.frame(fit2[[2]])
            bestindex<-as.numeric(rownames(Pareto2[1,]))
            yestimate<-predictions[,as.numeric(rownames(Pareto2[1,]))]}
          # PLOT FOR FIT2
          {plot(y,x, type="l", ylim=c(min(min(yestimate),min(x)), max(max(yestimate),max(x))))
            lines(y,yestimate, col=2)
          }
        }
      }
      
      #Inventory with Sales Model without Noise
      {
        #Read Data
        {
          data1<-t(data.table(read.csv("SimplifiedInventoryModel.csv",header=T),header=T))
          colnames(data1) = data1[1, ]
          data1 = data1[-1, ]
          data1=data1[-nrow(data1),]
          row.names(data1) <- 1:nrow(data1)
          #View(data1)
          data1<-data.frame(data1)
          
          data1$Replenishment <- as.numeric(levels(data1$Replenishment))[data1$Replenishment]
          data1$Inventory <- as.numeric(levels(data1$Inventory))[data1$Inventory]
          data1$Sales<-as.numeric(levels(data1$Sales))[data1$Sales]
          data1$Discrepancy <- as.numeric(levels(data1$Discrepancy))[data1$Discrepancy]
          
          x<-data1$Replenishment
          y<- data1$Inventory
          z<-data1$Sales
          t<-data1$Discrepancy
        }
        #Applying Curve Fitting for single cause variable
        {
          
            fit1<- curvefit(x=y,y=t)
            fit2<- curvefit(x=y,y=z)
            #FOR FIT1
            {results<- as.data.frame(fit1[[1]])
              results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
              results$Sigma<- round(results$Sigma, digits= 5)
              results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
              results$Mape<- round(results$Mape, digits= 5)
              
              Pareto1 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
              Pareto1
              # all<-data.frame(Sigma=Pareto1$Sigma,Mape=Pareto1$Mape,Method=Pareto1$Method,Function=Pareto1$Function, Relation=Pareto1$Relation)
              # all
              # best<-data.frame(Sigma=Pareto1[1,]$Sigma,Mape=Pareto1[1,]$Mape,Method=Pareto1[1,]$Method,Function=Pareto1[1,]$Function, Relation=Pareto1[1,]$Relation)
              # best
              
              predictions<- as.data.frame(fit1[[2]])
              bestindex<-as.numeric(rownames(Pareto1[1,]))
              yestimate<-predictions[,as.numeric(rownames(Pareto1[1,]))]
              yestimate<-as.matrix(yestimate)
            }
            # PLOT for FIT1
            { plot(x,y, type="l", ylim=c(min(min(yestimate[,1]),min(y)), max(max(yestimate[,1]),max(y))))
              lines(x,yestimate[,1], col=2)
            }
            #FOR FIT2
            { results<- as.data.frame(fit2[[1]])
              results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
              results$Sigma<- round(results$Sigma, digits= 5)
              results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
              results$Mape<- round(results$Mape, digits= 5)
              
              Pareto2 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
              Pareto2
              # all<-data.frame(Sigma=Pareto2$Sigma,Mape=Pareto2$Mape,Method=Pareto2$Method,Function=Pareto2$Function, Relation=Pareto2$Relation)
              # all
              # best<-data.frame(Sigma=Pareto2[1,]$Sigma,Mape=Pareto2[1,]$Mape,Method=Pareto2[1,]$Method,Function=Pareto2[1,]$Function, Relation=Pareto2[1,]$Relation)
              # best
              
              predictions<- as.data.frame(fit2[[2]])
              bestindex<-as.numeric(rownames(Pareto2[1,]))
              yestimate<-predictions[,as.numeric(rownames(Pareto2[1,]))]}
            # PLOT FOR FIT2
            {plot(y,x, type="l", ylim=c(min(min(yestimate),min(x)), max(max(yestimate),max(x))))
              lines(y,yestimate, col=2)
            }
          
        }
        #Applying SEM for multiple cause variables
        {
          #Convergence problem with the followig model1
          model1 <- '
          # regressions
          z ~ 0*1+ a*y
          t~ 1*500+b*y
          x ~ z+y
          # constraints
          a==0.05
          b==-1
          '
          training<- as.matrix(cbind(y,z,x,t))
          fit <- sem(model1, data=training)  
          summary(fit, standardized=TRUE)
        }
      }
      
      #Inventory with Sales Model with Noise
      {
        #Read Data
        {
          data1<-t(data.table(read.csv("NoisyInventoryModel.csv",header=T),header=T))
          colnames(data1) = data1[1, ]
          data1 = data1[-1, ]
          data1=data1[-nrow(data1),]
          row.names(data1) <- 1:nrow(data1)
          #View(data1)
          data1<-data.frame(data1)
          
          data1$Replenishment <- as.numeric(levels(data1$Replenishment))[data1$Replenishment]
          data1$Inventory <- as.numeric(levels(data1$Inventory))[data1$Inventory]
          data1$Sales<-as.numeric(levels(data1$Sales))[data1$Sales]
          data1$Discrepancy <- as.numeric(levels(data1$Discrepancy))[data1$Discrepancy]
          
          x<-data1$Replenishment
          y<- data1$Inventory
          z<-data1$Sales
          t<-data1$Discrepancy
        }
        #Applying Curve Fitting for single cause variable
        {
          
          fit1<- curvefit(x=y,y=t)
          fit2<- curvefit(x=y,y=z)
          #FOR FIT1
          {results<- as.data.frame(fit1[[1]])
            results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
            results$Sigma<- round(results$Sigma, digits= 5)
            results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
            results$Mape<- round(results$Mape, digits= 5)
            
            Pareto1 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
            Pareto1
            # all<-data.frame(Sigma=Pareto1$Sigma,Mape=Pareto1$Mape,Method=Pareto1$Method,Function=Pareto1$Function, Relation=Pareto1$Relation)
            # all
            # best<-data.frame(Sigma=Pareto1[1,]$Sigma,Mape=Pareto1[1,]$Mape,Method=Pareto1[1,]$Method,Function=Pareto1[1,]$Function, Relation=Pareto1[1,]$Relation)
            # best
            
            predictions<- as.data.frame(fit1[[2]])
            bestindex<-as.numeric(rownames(Pareto1[1,]))
            yestimate<-predictions[,as.numeric(rownames(Pareto1[1,]))]
            yestimate<-as.matrix(yestimate)
          }
          # PLOT for FIT1
          { plot(x,y, type="l", ylim=c(min(min(yestimate[,1]),min(y)), max(max(yestimate[,1]),max(y))))
            lines(x,yestimate[,1], col=2)
          }
          #FOR FIT2
          { results<- as.data.frame(fit2[[1]])
            results$Sigma<- as.numeric(levels(results$Sigma))[results$Sigma]
            results$Sigma<- round(results$Sigma, digits= 5)
            results$Mape<- as.numeric(levels(results$Mape))[results$Mape]
            results$Mape<- round(results$Mape, digits= 5)
            
            Pareto2 <- psel(results, low(results$Sigma) * low(results$Mape), top=min(10,nrow(results))) 
            Pareto2
            # all<-data.frame(Sigma=Pareto2$Sigma,Mape=Pareto2$Mape,Method=Pareto2$Method,Function=Pareto2$Function, Relation=Pareto2$Relation)
            # all
            # best<-data.frame(Sigma=Pareto2[1,]$Sigma,Mape=Pareto2[1,]$Mape,Method=Pareto2[1,]$Method,Function=Pareto2[1,]$Function, Relation=Pareto2[1,]$Relation)
            # best
            
            predictions<- as.data.frame(fit2[[2]])
            bestindex<-as.numeric(rownames(Pareto2[1,]))
            yestimate<-predictions[,as.numeric(rownames(Pareto2[1,]))]}
          # PLOT FOR FIT2
          {plot(y,x, type="l", ylim=c(min(min(yestimate),min(x)), max(max(yestimate),max(x))))
            lines(y,yestimate, col=2)
          }
          
        }
        #Applying SEM for multiple cause variables
        {
          #Convergence problem with the followig model1
          model2 <- '
          # regressions
          z ~ 0*1+ a*y
          t~ 1*500+b*y
          x ~ c*z+y
          # constraints
          a==0.05
          b==-1
          c==1
          '
          training<- as.matrix(cbind(y,z,x,t))
          fit <- sem(model2, data=training)  
          summary(fit, standardized=TRUE)
        }
      }
    }
  }
}