
library(nlme)
library(plyr)
library(data.table)
library(purrr)

source("xy curve fit.R")
source("xdy curve fit.R")
source("dxdy curve fit.R")

checkRelation<- function(k){
  #This function add a new column to the variable k with respect to the fitted
  #formula. It is used in curvefitAll function to be able to seperate different fits
  #from each other more easily.
  a<-grepl("dx",as.character(k$Function))
  if(a){
    k<- data.frame(k, Relation= "DXtoDY")
  }
  else{
    b<-grepl("dy",as.character(k$Function))
    if(b){
      k<- data.frame(k, Relation= "XtoDY")
    }
    else{
      k<- data.frame(k, Relation= "XtoY")
    }}
  return(k)
}
curveFitAll<- function(x1,y1,dx1=NA,dy1=NA){
  #curveFitAll is a function that fits different functions between inputs x1 and y1.
  # If rate of change of x1 and y1 is not given,namely dx1 and dy1, then dx1 and dy1
  # are calculated by substracting consecutive values in the data with the assumption
  # that DT=1. The main important part of curveFitAll is that it fits function not only
  # between x1 and y1 but also between x1 and dy1; dx1 and dy1. After fitting the curves,
  # function calculates RMSE and MAPE measures.
  # curveFitAll returns overallresults ([[1]]) and overallpredictions([[2]]).
  # Overallresults is the list of results returned by "curvefit()", "curvefitxdy()",
  # "curvefitdxdy". Overallpredictions is the list of predictions returned 
  # by "curvefit()", "curvefitxdy()","curvefitdxdy".
  
  if(is.na(dx1)){
    dx1<- lapply(1:(length(x1)), function(i) (x1[i+1]-x1[i]))  
    dx1<- dx1[-length(dx1)]
    dx1<- apply( cbind( dx1 ) , 1 , unlist )}
  
  if(is.na(dy1))
  { dy1<- lapply(1:(length(y1)), function(i) (y1[i+1]-y1[i]))  
    dy1<- dy1[-length(dy1)]
    dy1<- apply( cbind( dy1 ) , 1 , unlist )}
  
  fitxy<- curvefit(x1,y1)
  fitxdy<- curvefitxdy(x1,y1,dy1)
  fitdxdy<- curvefitdxdy(dx1,y1,dy1)
  
  colnames(fitxdy[[2]])<- if(nrow(fitxdy[[1]])>0){paste0("XDY",colnames(fitxdy[[2]]) ) }
  colnames(fitdxdy[[2]])<- if(nrow(fitdxdy[[1]])>0){ paste0("DXDY",colnames(fitdxdy[[2]]) )}
  
  overallresults<- rbind.fill(fitxy[[1]],fitxdy[[1]],fitdxdy[[1]]) 
  listedresults<-lapply(1:nrow(overallresults), function(i) checkRelation(overallresults[i,]))
  overallresults<- apply( t(rbind.fill( listedresults )) , 1 , unlist )
  
  overallpredictions<- NULL
  if(nrow(fitxy[[1]])>0)
  {overallpredictions<-cbind(overallpredictions, fitxy[[2]])}
  if(nrow(fitxdy[[1]])>0)
  {overallpredictions<-cbind(overallpredictions, fitxdy[[2]])}
  if(nrow(fitdxdy[[1]])>0)
  {overallpredictions<-cbind(overallpredictions, fitdxdy[[2]])}
   
  outputall<- list(overallresults,overallpredictions)
  return(outputall)
}

