library(rPref)

curvefit<- function(x,y){
  results<-data.frame()
  predictions<-NULL
  
  #linear models
  m1<-NULL
  m1<- tryCatch(gnls(y~ a+ b*x , start= c(a=0, b=1)), error=function(e) NULL)
  if(!is.null(m1)){results<- rbind.fill(results, data.frame(cbind(rbind(m1$coefficients),Mape=100*sum(abs((y-predict(m1))/y))/length(y), Sigma=sqrt(sum((y-predict(m1))^2)/length(y)), Method= "LinFunc", Function="y~ a+b*x", Model= "m1", Isnull=is.null(m1) ) ))
  predictions<- cbind(predictions, M1=predict(m1))}
  
  m2<-NULL
  m2<- tryCatch(lm(y~ x), error=function(e) NULL)
  summary(m2)
  if(!is.null(m2)){results<- rbind.fill(results, data.frame(cbind(rbind(m2$coefficients),Mape=100*sum(abs((y-predict(m2))/y))/length(y), Sigma=sqrt(sum((y-predict(m2))^2)/length(y)), Method= "LinFunc", Function="y~ a+b*x", Model= "m2", Isnull=is.null(m2) ) ))
  predictions<- cbind(predictions, M2=predict(m2))}
  

  #exponential and s-shaped models 
  m3<- NULL
  logy<- log(y)
  m3<- tryCatch(gnls(logy~ a+b*x, start=c(a=0, b=1)), error=function(e) NULL)
  if(!is.null(m3)){results<- rbind.fill(results, data.frame(cbind(rbind(m3$coefficients),Mape=100*sum(abs((y-exp(predict(m3)))/y))/length(y), Sigma=sqrt(sum((y-exp(predict(m3)))^2)/length(y)), Method= "ExpFunc", Function="logy~a+b*x", Model= "m3", Isnull=is.null(m3)  ) ))
  predictions<- cbind(predictions, M3=exp(predict(m3)))}
  
  m7<- NULL
  m7<- tryCatch(nls(y~ SSlogis(x, Asym, xmid, scal)), error=function(e) NULL)
  if(!is.null(m7)){results<- rbind.fill(results, data.frame(cbind(rbind(coef(m7)),Mape=100*sum(abs((y-predict(m7))/y))/length(y), Sigma=sqrt(deviance(m7)), Method= "SShaped", Function="y~ SSlogis(x, Asym, xmid, scal)", Model= "m7", Isnull=is.null(m7)  ) ))
  predictions<- cbind(predictions, M7=predict(m7))}
  
  m8<- NULL
  m8<-  tryCatch(nls(y~ SSasymp(x, Asym, R0, lrc)), error=function(e) NULL)
  if(!is.null(m8)){results<- rbind.fill(results, data.frame(cbind(rbind(coef(m8)),Mape=100*sum(abs((y-predict(m8))/y))/length(y), Sigma=sqrt(deviance(m8)), Method= "SShaped", Function="y~ SSasymp(x, Asym, R0, lrc)", Model= "m8", Isnull=is.null(m8)  ) ))
  predictions<- cbind(predictions, M8=predict(m8))}
  
  m11<- NULL
  logy<- log(-y)
  m11<- tryCatch(gnls(logy~ a+b*x, start=c(a=0, b=1)), error=function(e) NULL)
  if(!is.null(m11)){results<- rbind.fill(results, data.frame(cbind(rbind(m11$coefficients),Mape=100*sum(abs((y+exp(predict(m11)))/y))/length(y), Sigma=sqrt(sum((y+exp(predict(m11)))^2)/length(y)), Method= "ExpFunc", Function="log-y~ a+b*x", Model= "m11", Isnull=is.null(m11)  ) ))
  predictions<- cbind(predictions, M11=-exp(predict(m11)))}
  

  if(!is.null(m7))
  {
    m13<- NULL
    m13<- tryCatch( gnls(y ~ Asym/(1+exp((xmid-x)/scal)), start = coef(m7)), error=function(e) NULL)
    if(!is.null(m13)){results<- rbind.fill(results, data.frame(cbind(rbind(m13$coefficients),Mape=100*sum(abs((y-predict(m13))/y))/length(y), Sigma=sqrt(sum((y-predict(m13))^2)/length(y)), Method= "SShaped", Function="y ~ Asym/(1+exp((xmid-x)/scal)", Model= "m13", Isnull=is.null(m13)  ) ))
    predictions<- cbind(predictions, M13=predict(m13))}
  }
  
  m16<- NULL
  m16<- tryCatch( gnls(y ~max(y)/(1+exp((xmid-x)/scal)), start = c(xmid=1, scal=1)), error=function(e) NULL)
  if(!is.null(m16)){results<- rbind.fill(results, data.frame(cbind(rbind(m16$coefficients),Mape=100*sum(abs((y-predict(m16))/y))/length(y), Sigma=sqrt(sum((y-predict(m16))^2)/length(y)), Method= "SShaped", Function="y ~ max(y)/(1+exp((xmid-x)/scal)", Model= "m16", Isnull=is.null(m16)  ) ))
  predictions<- cbind(predictions, M16=predict(m16))}
  
  m17<- NULL
  m17<- tryCatch( gnls(y ~max(y)-(max(y)-min(y))/(1+exp((xmid-x)/scal)), start = c(xmid=1, scal=1)), error=function(e) NULL)
  if(!is.null(m17)){results<- rbind.fill(results, data.frame(cbind(rbind(m17$coefficients), Mape=100*sum(abs((y-predict(m17))/y))/length(y),Sigma=sqrt(sum((y-predict(m17))^2)/length(y)), Method= "SShaped", Function="y ~max(y)-(max(y)-min(y))/(1+exp((xmid-x)/scal))", Model= "m17", Isnull=is.null(m17)  ) ))
  predictions<- cbind(predictions, M17=predict(m17))}
  
  m18<- NULL
  m18<- tryCatch( gnls(y ~min(y) +(max(y)-min(y))/(1+exp((xmid-x)/scal)), start = c(xmid=1, scal=1)), error=function(e) NULL)
  if(!is.null(m18)){results<- rbind.fill(results, data.frame(cbind(rbind(m18$coefficients),Mape=100*sum(abs((y-predict(m18))/y))/length(y), Sigma=sqrt(sum((y-predict(m18))^2)/length(y)), Method= "SShaped", Function="y ~ min(y) +(max(y)-min(y))/(1+exp((xmid-x)/scal)", Model= "m18", Isnull=is.null(m18)  ) ))
  predictions<- cbind(predictions, M18=predict(m18))}
  
  results<- results[which(results$Isnull == FALSE),]
  results1<-results
  predictions1<-predictions
  output1<-list(results1, predictions1)
  return (output1)
}